import time
import pygame

from base import config, ticks
from events import process_input

from textscroll import TextScroller

def build_logo(screen, scene, logo):
    picture_opt = 'logo_%s_picture' % logo
    pos_opt = 'logo_%s_pos' % logo
    picture = pygame.image.load(config.get(scene, picture_opt))
    #picture = picture.convert_alpha()
    rect = picture.get_rect()
    pos_name = config.get(scene, pos_opt)
    #print "logo rect", rect
    screen_width = screen.get_width()
    logo_width = rect.width
    screen_height = screen.get_height()
    logo_height = rect.height
    if pos_name == 'top-center':
        #print "using top-center"
        x = (screen_width - logo_width) / 2
        y = 0
    elif pos_name == 'bottom-center':
        x = (screen_width - logo_width) / 2
        y = screen_height - logo_height
    pos = (x,y)
    return picture, pos

        

class VideoWidget(object):
    def __init__(self, scene):
        self.scene = scene
        self.screen = None
        self.screen_size = None
        self.movie = None
        self.rect = None
        
    def load_movie(self, filename=None):
        if filename is None:
            filename = config.get(self.scene, 'video_file')
        self.movie = pygame.movie.Movie(filename)

    def set_screen(self, screen):
        self.screen = screen
        self.screen_size = self.screen.get_size()
        
    def prepare_movie(self, rect=None):
        self.rect = rect
        print "rect is", self.rect
        if self.rect is None:
            self.rect = config.get_xy(self.scene, 'video_rect')
        print "rect is", self.rect
        #import pdb
        #pdb.set_trace()
        self.movie.set_display(self.screen, self.rect)

class VideoBackground(VideoWidget):
    def __init__(self, screen, scene):
        VideoWidget.__init__(self, scene)
        self.set_screen(screen)
        
    def prepare_movie(self):
        rect = self.screen.get_rect()
        VideoWidget.prepare_movie(rect)
        
class Scene(object):
    def __init__(self, screen, scene):
        self.screen = screen
        self.screen_size = screen.get_size()
        self.scene = scene
        self.logos = {}
        self.video = None
        # not using the clock yet
        self.clock = pygame.time.Clock()
        timer_style = config.get(self.scene, 'timer_style')
        if timer_style == 'sleep':
            self.delay_function = pygame.time.wait
        else:
            self.delay_function = pygame.time.delay
        self.text_scroll = None
        
    def initialize(self):
        self.create_background()
        self.create_logos()
        self.create_video()
        self.text_scroll = self.create_text_scroll()
        self.initialize_textscroll_position()
        
    def create_background(self):
        background_filename = config.get(self.scene, 'background')
        background = pygame.image.load(background_filename)
        background = background.convert()
        background = pygame.transform.scale(background, self.screen_size)
        self.background = background

    def create_video(self):
        filename = config.get(self.scene, 'video_file').strip()
        if not filename:
            return
        self.video = VideoWidget(self.scene)
        self.video.load_movie(filename=filename)
        self.video.set_screen(self.screen)
        self.video.prepare_movie()
        
    def create_text_scroll(self):
        # get the text to be scrolled first
        text = config.get(self.scene, 'text')
        # test to see if there's any text for the scroller
        # and return 'None' if there's no text (could just
        # use return, but I want to make the return value
        # explicit, so we can remember we're using it)
        if not text.strip():
            #return None

            # I changed my mind, for now.  Since we can't
            # perform the scene without a text_scroll, we're
            # just going to make a "whitespace scroller".
            # FIXME:  Implement either a hard time for a scene
            # without a text_scroll to be displayed, or base it
            # on the playing of a video.
            text = ' '
        # at this point, there should be text to scroll, so
        # we create the scroller
        text_scroll = TextScroller()
        text_scroll.set_screen(self.screen)
        text_scroll.set_background(self.background)
        font_size = config.getint(self.scene, 'font_size')
        font_file = config.get(self.scene, 'font_file').strip()
        if font_file:
            text_scroll.set_font_file(font_file, font_size)
        else:
            text_scroll.set_sysfont('Arial', font_size)
        
        text_scroll.set_text(text)

        color_name = config.get(self.scene, 'font_color')
        color = config.get_color(self.scene, 'font_color')
        text_scroll.set_color(color)

        speed = config.get_speed(self.scene)
        text_scroll.set_speed(speed)

        # use logo if defined
        # allows use of premade text saved as image
        # to scroll across the screen
        logo = config.get(self.scene, 'text_logo').strip()
        if logo:
            surface = pygame.image.load(logo)
            surface = surface.convert_alpha()
            text_scroll.surface = surface
        else:
            text_scroll.make_surface()
        return text_scroll

    def create_logos(self):
        logos = config.get_list(self.scene, 'static_logos')
        for logo in logos:
            picture, pos = build_logo(self.screen, self.scene, logo)
            self.logos[logo] = picture, pos
            
            
        
    def initialize_textscroll_position(self, text_scroll=None):
        if text_scroll is None:
            text_scroll = self.text_scroll
        # if there's not text_scroll, we do nothing
        if text_scroll is None:
            return
        width, height = self.screen_size
        rect = text_scroll.surface.get_rect()
        start_pos = (width, height - (rect.bottom - rect.top))
        text_scroll.rect = rect.move(start_pos)

    # this method needs to be updated to handle the
    # absence of a text_scroll.  Currently, all the scenes
    # are timed and performed based on the text_scroll
    # moving across the screen.  Without a text_scroll object,
    # we are unable to really perform a scene.
    def perform_scene(self):
        self.initialize_textscroll_position()
        self.text_scroll.finished = False
        delay = config.getint(self.scene, 'delay')
        self.screen.blit(self.background, (0, 0))
        logo_rects = []
        for pict, pos in self.logos.values():
            self.screen.blit(pict, pos)
            rect = pict.get_rect().move(pos)
            logo_rects.append(rect)
        if self.video is not None:
            self.video.movie.play()
        pygame.display.flip()
        while not self.text_scroll.finished:
            rects = self.text_scroll.scroll()
            for rect in rects:
                self.screen.blit(self.background, rect, rect)
            for pict, pos in self.logos.values():
                self.screen.blit(pict, pos)
            update_rects = list(rects)
            self.screen.blit(self.text_scroll.surface, self.text_scroll.rect)
            pygame.display.update(update_rects)
            if self.video is not None:
                if not self.video.movie.get_busy():
                    self.video.movie.rewind()
                    self.video.movie.play()
            self.delay_function(delay)
            events = pygame.event.get()
            process_input(events)
            
